#pragma implementation
#include <stdio.h>
#include <ctype.h>
#include <tlmpweb.h>
#include "tlmpdoc.h"

static W_INT step1("s1");
static W_INT step2("s2");
static W_INT step3("s3");
static W_INT vdepth("dp");	// Depth at which we start presenting several
							// section
static W_INT content("tb");	// Present the table of content
static int tbstep[3];

void page(_F_page &c, const char *title)
{
	printf ("<h2>%s</h2><p>\n",title);
	c.desc();
	c.image();
}

static bool is_index (int niveau, int tbno[])
{
	bool ret = true;
	for (int i=0; i<niveau; i++){
		if (tbstep[i] != tbno[i]){
			ret = false;
			break;
		}
	}
	if (ret){
		ret = tbstep[niveau] == 0;
	}
	return ret;
}

static bool is_data (int niveau, int tbno[])
{
	bool ret = true;
	for (int i=0; i<=niveau; i++){
		if (tbstep[i] != tbno[i]){
			ret = false;
			break;
		}
	}
	return ret;
}

static bool is_step (int s1, int s2, int s3)
{
	return tbstep[0] == s1
		&& tbstep[1] == s2
		&& tbstep[2] == s3;
}

static int tbno[10];
static int niveau=0;

static void tlmpdoc_table_entry (const char *title)
{
	tlmpweb_ctrloutput (true);
	for (int i=0; i<niveau; i++) printf ("....");
	printf ("<a href=%s?s1=%d&s2=%d&s3=%d>%s</a><br>\n"
		,tlmpweb_curpage()
		,tbno[0],tbno[1],tbno[2]
		,title);
	tlmpweb_ctrloutput (false);
}


void section(_F_section &c, const char *title, bool last)
{
	c.priv = NULL;
	tbno[niveau]++;
	if (content != 0){
		tlmpdoc_table_entry (title);
		niveau++;
		tbno[niveau] = 0;
		c.body();
		tbno[niveau] = 0;
		niveau--;
	}else if (vdepth <= niveau){
		niveau++;
		printf ("<h%d>%s</h%d><p>\n",niveau,title,niveau);
		c.body();
		niveau--;
	}else{
		int here = tbno[niveau];
		if (is_index (niveau,tbno)){
			bool on = tlmpweb_ctrloutput (true);
			if (here == 1) printf ("<ul>\n");
			printf ("<li><a href=%s",tlmpweb_curpage());
			char sep = '?';
			for (int i=0; i<3;  i++){
				printf ("%cs%d=%d",sep,i+1,tbno[i]);
				sep = '&';
			}
			printf (">%s</a>\n",title);
			tlmpweb_ctrloutput (on);
		}else if (is_data(niveau,tbno)){
			niveau++;
			tbno[niveau] = 0;
			bool on = tlmpweb_ctrloutput (tbstep[niveau] == 0);
			tlmpweb_title (title);
			tlmpweb_body  ("white","red");
			printf ("<center><table border=0><tr bgcolor=lightblue>\n");
			printf ("<td>\n");
			printf ("<a href=%s>Top</a><br>\n",tlmpweb_curpage());
			printf ("<a href=%s?tb=1>Content</a>\n",tlmpweb_curpage());
			printf ("<td>\n");
			{
				printf ("<a href=%s",tlmpweb_curpage());
				char sep = '?';
				for (int i=0; i<3;  i++){
					printf ("%cs%d=%d",sep,i+1,i > niveau-2 ? 0 : tbno[i]);
					sep = '&';
				}
				printf (">Up</a>\n");
			}
			if (here > 1){
				if (niveau > 1) printf ("<br>\n");
				printf ("<a href=%s",tlmpweb_curpage());
				char sep = '?';
				for (int i=0; i<3;  i++){
					printf ("%cs%d=%d",sep,i+1,i==niveau-1 ? here-1 : tbno[i]);
					sep = '&';
				}
				printf (">Prec.</a>\n");
			}
			printf ("<td width=90%%><center><h1>%s</h1></center>\n",title);
			printf ("<td>\n");
			if (!last){
				printf ("<a href=%s",tlmpweb_curpage());
				char sep = '?';
				for (int i=0; i<3;  i++){
					printf ("%cs%d=%d",sep,i+1,i==niveau-1 ? here+1 : tbno[i]);
					sep = '&';
				}
				printf (">Next</a>\n");
			}
			printf ("</table></center>\n");
			printf ("<p>\n");
			printf ("<center><table border=0 width=80%%><tr><td>\n");
			c.body();
			printf ("</td></table></center>\n");
			tlmpweb_ctrloutput (on);
			niveau--;
		}
	}
}

void section(_F_section &c, const char *title)
{
	section (c,title,false);
}
	
void showimage (const char *fname)
{
	printf ("<center><img src=images/%s.jpeg></center>\n",fname);
}

void paragraph(_F_paragraph &c, const char *title)
{
	c.priv = NULL;
	if (content != 0){
		tlmpdoc_table_entry (title);
	}else{
		printf ("<h3>%s</h3><p>\n",title);
		printf ("<dl><dd>\n");
		c.body();
		printf ("<p></dl>\n");
	}
}



void document(_F_document &c, const char *title)
{
	c.priv = NULL;
	tbstep[0] = step1;
	tbstep[1] = step2;
	tbstep[2] = step3;
	if (!vdepth.isset()) vdepth = 20;
	tlmpweb_title (title);
	if (content != 0){
		printf ("<center><h1>%s</h1></center><p>\n",title);
		printf ("<a href=%s>Top</a><br>\n",tlmpweb_curpage());
		printf ("<strong>Table of content</strong><p>\n");
		printf ("<dl><dd>\n");
		tlmpweb_ctrloutput (false);
		c.body();
		printf ("</dl>\n");
	}else{
		if (is_step (0,0,0)){
			printf ("<center><h1>%s</h1></center><p>\n",title);
			printf ("<center><table width=70%%><tr><td>\n");
			c.abstract();
			printf ("</td></table></center><p>\n");
			printf ("<center><table border=0><tr><td>\n");
		}
		c.body();
		if (is_step(0,0,0)){
			printf ("</table></center>\n");
			printf ("<a href=%s?tb=1>Table of content</a><br>\n"
				,tlmpweb_curpage());
			if (vdepth == 20){
				printf ("<a href=%s?dp=0>Full text</a><br>\n"
					,tlmpweb_curpage());
			}else{
				printf ("<a href=%s>Top</a><br>\n",tlmpweb_curpage());
			}
		}
	}
}
	
struct _F_sample_private{
	SSTRINGS tb;
};

void _F_sample::fputs (const char *s, FILE *)
{
	priv->tb.add (new SSTRING(s));
}

void sample (_F_sample &c, const char *title, int width)
{
	if (content != 0){
		tlmpdoc_table_entry (title);
	}else if (is_data(niveau,tbno) || vdepth <= niveau){
		_F_sample_private priv;
		c.priv = &priv;
		c.body();
		// We remove the space in front of the lines
		int n = priv.tb.getnb();
		int minspaces=1000;
		for (int i=0; i<n; i++){
			const char *s = priv.tb.getitem(i)->get();
			int spaces = 0;
			while (isspace(*s)){
				spaces++;
				s++;
			}
			if (*s > ' '){
				if (spaces < minspaces) minspaces = spaces;
			}
		}
		printf ("<br>\n");
		printf ("<center><table border=1 width=%d%%><tr><td><pre>\n",width);
		for (int i=0; i<n; i++){
			const char *s = priv.tb.getitem(i)->get();
			int spaces = 0;
			while (spaces < minspaces && isspace(*s)){
				spaces++;
				s++;
			}
			fputs (s,stdout);
		}
		printf ("</td></tr></table></pre>\n");
		printf ("<br><strong>%s</strong></center>\n",title);
		printf ("<br>\n");
	}
}

void sample (_F_sample &c, const char *title)
{
	sample (c,title,60);
}

