#include <string.h>
#include <stdarg.h>
#include <misc.h>
#include "tlmpweb.h"

static W_INT formdo ("formdo");

// Hidden variables to carry with the form
class FORM_HIDDENS: public ARRAY{
	/*~PROTOBEG~ FORM_HIDDENS */
public:
	FORM_HIDDEN *getitem (int no)const;
	/*~PROTOEND~ FORM_HIDDENS */
};

PUBLIC FORM_HIDDEN *FORM_HIDDENS::getitem (int no) const
{
	return (FORM_HIDDEN*)ARRAY::getitem(no);
}

static FORM_HIDDENS hiddens;

PUBLIC FORM_HIDDEN::FORM_HIDDEN(W_VAR &_var)
	: var(_var)
{
	hiddens.neverdelete();
	hiddens.add (this);
}

PUBLIC FORM_HIDDEN::~FORM_HIDDEN()
{
	hiddens.remove (this);
}

PUBLIC const char *FORM_HIDDEN::getname()
{
	return var.getname();
}
PUBLIC const char *FORM_HIDDEN::getval()
{
	var.getvalstr (s);
	return s.get();
}

/*
	Get the list of hidden parameter to form URLs
*/
void form_gethidden (SSTRING &s)
{
	s.setfrom("");
	const char *sep = "";
	for (int i=0; i<hiddens.getnb(); i++){
		FORM_HIDDEN *h = hiddens.getitem(i);
		const char *val = h->getval();
		if (val[0] != '\0'){
			s.appendf ("%s%s=%s",sep,h->getname(),val);
			sep = "&";
		}
	}
}
		


class FORM_INFO{
	/*~PROTOBEG~ FORM_INFO */
	/*~PROTOEND~ FORM_INFO */
};
enum FORM_STATUS {FORM_NOTHING, FORM_SETUP, FORM_VALIDATE, FORM_PROCESS};

struct _F_form_private{
	FORM_INFO info;
	FORM_STATUS status;
	SSTRING lastname;
	SSTRING lastvalue;
	bool submit;
};



static const char *form_geteditval(
	const char *name,
	const char *value)
{
	if (t_var_isdef(name)){
		// We are re-editing after an error, so use the value
		// previously entered by a user
		value = t_varstr(name);
	}
	return value;
}


PUBLIC _F_form::_F_form()
{
	priv = new _F_form_private;
	priv->status = FORM_NOTHING;
}

PUBLIC void _F_form::fputs (const char *s, FILE *f)
{
	if (f == stdout){
		if (priv->status == FORM_SETUP || priv->status == FORM_VALIDATE){
			::fputs (s,f);
		}
	}else{
		::fputs (s,f);
	}
}

PUBLIC void _F_form::field_radio (
	const char *name,
	const char *value,
	bool selected,
	const char *opt)
{
	if (t_var_isdef(name)){
		// We are re-editing after an error, so use the value
		// previously entered by a user
		const char *val = t_varstr(name);
		selected = strcmp(val,value) == 0;
	}
	char tmpopt[strlen(opt)+9];
	sprintf (tmpopt,"%s %s",selected ? "checked" : "", opt);
	setfield (name,"radio",value,tmpopt);
}
PUBLIC void _F_form::field_checkbox (
	const char *name,
	bool selected,
	const char *opt)
{
	if (t_var_isdef(name)){
		// We are re-editing after an error, so use the value
		// previously entered by a user
		selected = t_varint(name) != 0;
	}
	char tmpopt[strlen(opt)+9];
	sprintf (tmpopt,"%s %s",selected ? "checked" : "", opt);
	setfield (name,"checkbox","1",tmpopt);
}
PUBLIC void _F_form::field_string (
	const char *name,
	const char *value,
	const char *opt)
{
	setfield (name,"text",value,opt);
}

PUBLIC void _F_form::field_string (
	const W_VAR &var,
	const char *opt)
{
	SSTRING s;
	var.getvalstr(s);
	setfield (var.getname(),"text",s.get(),opt);
}

PUBLIC void _F_form::field_password (
	const char *name,
	const char *opt)
{
	setfield (name,"password","",opt);
}

PUBLIC void _F_form::field_password (
	const W_VAR &var,
	const char *opt)
{
	setfield (var.getname(),"password","",opt);
}

PUBLIC void _F_form::field_hidden (
	const char *name,
	const char *value)
{
	setfield (name,"hidden",value,"");
}

PUBLIC void _F_form::field_textarea (
	const char *name,
	int rows,
	int cols,
	const char *opt)
{
	priv->lastname.setfrom (name);
	if (priv->status == FORM_SETUP){
		::printf ("<textarea name=%s rows=%d cols=%d>\n",name,rows,cols);
		if (t_var_isdef(name)){
			::printf ("%s",t_varstr(name));
		}
	}
}

PUBLIC void _F_form::field_textarea_body(const char *ctl, ...)
{
	if (priv->status == FORM_SETUP && !t_var_isdef(priv->lastname.get())){
		va_list list;
		va_start (list,ctl);
		vprintf (ctl,list);
		va_end (list);
	}
}
PUBLIC void _F_form::field_textarea_end()
{
	if (priv->status == FORM_SETUP){
		::printf ("</textarea>\n");
	}
}

PUBLIC void _F_form::field_textarea (
	const W_VAR &var,
	int rows,
	int cols,
	const char *opt)
{
	field_textarea (var.getname(),rows,cols,opt);
	SSTRING s;
	var.getvalstr(s);
	field_textarea_body ("%s",s.get());
	field_textarea_end ();
}

PUBLIC void _F_form::field_list (
	const char *name,
	const char *value,
	const char *opt)
{
	if (priv->status == FORM_SETUP){
		::printf ("<select name=%s>\n",name);
		priv->lastvalue.setfrom (form_geteditval(name,value));
	}
}

PUBLIC void _F_form::field_list_item (
	const char *sel_value,
	const char *sel_desc)
{
	if (priv->status == FORM_SETUP){
		printf ("<option value=\"%s\" %s>%s\n",sel_value
			,priv->lastvalue.cmp(sel_value)==0 ? "selected" : ""
			,sel_desc);
	}
}

PUBLIC void _F_form::field_list_item (
	const char *sel_value)
{
	field_list_item(sel_value,sel_value);
}


PUBLIC void _F_form::field_list_end()
{
	if (priv->status == FORM_SETUP){
		::printf ("</select>\n");
	}
}

PUBLIC void _F_form::field_file (const char *name)
{
	printf ("<input type=file name=%s>\n",name);
}

PUBLIC void _F_form::field_file (W_VAR &v)
{
	field_file (v.getname());
}


PUBLIC void _F_form::printf (const char *ctl, ...)
{
	if (priv->status == FORM_SETUP || priv->status == FORM_VALIDATE){
		va_list list;
		va_start (list,ctl);
		vprintf (ctl,list);
		va_end (list);
	}
}

PUBLIC void _F_form::setfield (
	const char *name,
	const char *type,
	const char *value,
	const char *opt)
{
	//tbvar->add (name
	if (priv->status == FORM_SETUP){
		value = form_geteditval(name,value);
		::printf ("<input type=%s name=%s value=\"%s\" %s>\n",type,name,value,opt);
	}
}

/*
	Set the submit button.
	This allow the placement of the submit button as needed. If this
	function is not called, the submit button will be placed at the end
	of the form.
*/
PUBLIC void _F_form::button_submit ()
{
	priv->submit = true;
	printf ("<input type=submit>\n");
}

/*
	Set the submit button with a custom title
	This allow the placement of the submit button as needed. If this
	function is not called, the submit button will be placed at the end
	of the form.
*/
PUBLIC void _F_form::button_submit (const char *title)
{
	priv->submit = true;
	printf ("<input type=submit value=\"%s\">\n",title);
}

/*
	Tell the form we do not want a submit button at all
*/
PUBLIC void _F_form::no_submit ()
{
	priv->submit = true;
}

static void setup (_F_form &c, bool first, bool drawing)
{
	c.priv->status = FORM_SETUP;
	c.priv->submit = false;
	printf ("<form action=%s method=post ENCTYPE=\"multipart/form-data\">\n",tlmpweb_curpage());
	printf ("<input type=hidden name=formdo value=1>\n");
	for (int i=0; i<hiddens.getnb(); i++){
		FORM_HIDDEN *h = hiddens.getitem(i);
		printf ("<input type=hidden name=%s value=\"%s\">\n",h->getname()
			,h->getval());
	}
	c.setup(first,drawing);
	if (!c.priv->submit) printf ("<input type=submit>\n");
	printf ("</form>\n");
}

PUBLIC VIRTUAL void _F_form::load(){}
PUBLIC VIRTUAL void _F_form::top(){}
PUBLIC VIRTUAL void _F_form::bottom(){}
void form (_F_form &c)
{
	if (formdo == 0){
		// Prepare the form
		c.load();
		c.priv->status = FORM_SETUP;
		c.top();
		setup (c,true,true);
		c.bottom();
	}else if (formdo == 1){
		c.priv->status = FORM_NOTHING;
		c.setup(false,false);		// Call this to learn the definition of the fields
		c.priv->status = FORM_VALIDATE;
		// We can do some pre validation based on the type. We will do this
		// one day.
		// ...
		if (c.validate()){
			formdo = 0;		// Allow other form in the same document
			c.process();
		}else{
			c.top();
			setup (c,false,true);
			c.bottom();
		}
	}else{
		tlmpweb_error ("Invalid formdo value\n");
	}
}

