#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <misc.h>
#include "tlmpweb.h"

static W_VAR *first;

static bool is_init = false;
/*
	Initialise the W_VAR variable from the HTTP request.
	Initialise only the static variable as this is called from tlmpweb
	main(). Other W_VAR variable will initialise themselves from
	their constructor. See is_init below.
*/
void wvar_init()
{
	W_VAR *pt = first;
	while (pt != NULL){
		if (!pt->isset()){	
			bool is_def;
			const char *val = t_varstr (pt->getname(),is_def);
			if (is_def) pt->setinitval (val);
		}
		pt = pt->getnext();
	}
	is_init = true;
}

W_VAR *wvar_getfirst ()
{
	return first;
}

PUBLIC W_VAR::W_VAR(const char *_name, const char *_flags)
{
	name = strdup(_name);
	next = first;
	first = this;
	was_set = false;
	if (_flags == NULL) _flags = "";
	flags = strdup(_flags);
}

/*
	For a variable called after the wvar_init() was called, we
	check if there is a value in the HTTP request.
*/
PROTECTED void W_VAR::after_init()
{
	if (is_init){
		bool is_def;
		const char *val = t_varstr (getname(),is_def);
		if (is_def) setinitval (val);
	}
}

PUBLIC VIRTUAL W_VAR::~W_VAR()
{
	free ((char*)name);
	free ((char*)flags);
	W_VAR **ptpt = &first;
	while (*ptpt != NULL){
		if (*ptpt == this){
			*ptpt = next;
			break;
		}
		ptpt = &(*ptpt)->next;
	}
}

PUBLIC W_VAR * W_VAR::getnext() const
{
	return next;
}
PUBLIC const char *W_VAR::getflags() const
{
	return flags;
}
/*
	Return true if the variable was initialised (was supplied by the request)
*/
PUBLIC bool W_VAR::isset()
{
	return was_set;
}

/*
	Reset the content of the variable received
*/
PUBLIC void W_VAR::setempty()
{
	t_var_setempty(name);
}	

PUBLIC const char *W_VAR::getname() const
{
	return name;
}

PUBLIC W_SSTRING::W_SSTRING(const char *_name)
	: SSTRING(), W_VAR(_name,"")
{
	after_init();
}

PUBLIC W_SSTRING::W_SSTRING(const char *_name, const char *value)
	: SSTRING(), W_VAR(_name,"")
{
	setfrom (value);
	after_init();
}

PUBLIC W_SSTRING::W_SSTRING(const char *_name, const char *value, const char *_flags)
	: SSTRING(), W_VAR(_name,_flags)
{
	setfrom (value);
	after_init();
}

PUBLIC void W_SSTRING::setinitval(const char *v)
{
	setfrom (v);
	was_set = true;
}

PUBLIC void W_SSTRING::getvalstr (SSTRING &s) const
{
	s.setfrom (get());
}


PUBLIC void W_SSTRING::setempty()
{
	SSTRING::setempty();
	W_VAR::setempty();
}


PUBLIC W_INT::W_INT(const char *_name)
	: W_VAR (_name,"")
{
	val = 0;
	after_init();
}
PUBLIC W_INT::W_INT(const char *_name, int value)
	: W_VAR (_name,"")
{
	val = value;
	after_init();
}
PUBLIC W_INT::W_INT(const char *_name, int value, const char *_flags)
	: W_VAR (_name,_flags)
{
	val = value;
	after_init();
}

PUBLIC void W_INT::setinitval(const char *v)
{
	if (isdigit(v[0])){
		was_set = true;
		val = atoi(v);
	}
}

PUBLIC  W_INT::operator int()
{
	return val;
}

PUBLIC W_INT & W_INT::operator =(int v)
{
	val = v;
	return *this;
}

PUBLIC void W_INT::getvalstr (SSTRING &s) const
{
	s.setfromf ("%d",val);
}

PUBLIC int W_INT::getval ()
{
	return val;
}

